const widgets = {}
const charts = {}
//const series = {}
const markers = {}
/*const linedatas = {}*/

// This function opens an indicator on the given chart.
function open_indicator(chartId, symbol, state, mobile_view, handler) {
    var oldState = null;
    var interval = "1";
    if (state != null) {
        oldState = JSON.parse(state);
        interval = oldState.charts[0].panes[0].sources[0].state.interval;
        oldState.charts[0].panes[0].sources[0].state.symbol = symbol;
    }
    const chart = new TradingView.widget({
        symbol,
        interval: interval,
        fullscreen: false,
        container: chartId,
        custom_css_url: 'chart_theme.css',
        library_path: '_content/BadgerWeb.Component/lib/charting_library/',
        theme: 'dark',
        saved_data: oldState,
        preset: mobile_view ? 'mobile' : null,
        overrides: {
            "paneProperties.backgroundType": "solid",
            "paneProperties.background": "#181A20",
            "paneProperties.vertGridProperties.color": "#181A20",
            "paneProperties.horzGridProperties.color": "#181A20",
            "scalesProperties.textColor": "#D3D3D3",
        },
        datafeed: {
            onReady: callback => {
                console.log('[onReady]: Method call');
                setTimeout(() => callback(configurationData));
            },
            searchSymbols: async (userInput, exchange, symbolType, onResultReadyCallback) => {
                try {
                    const dataset = await handler.invokeMethodAsync('SearchSymbol', userInput);
                    onResultReadyCallback(dataset);
                } catch (error) {
                    console.error("Error during searchSymbols:", error);
                }
            },
            resolveSymbol: async (symbolName, onSymbolResolvedCallback, onResolveErrorCallback, ext) => {
                try {
                    const symbolItem = await handler.invokeMethodAsync('ResolveSymbol', symbolName);
                    if (!symbolItem) {
                        onResolveErrorCallback('cannot resolve symbol');
                        return;
                    }
                    const symbolInfo = {
                        ticker: symbolItem.ticker,
                        name: symbolItem.symbol,
                        description: symbolItem.description,
                        type: symbolItem.type,
                        session: '24x7',
                        timezone: 'Etc/UTC',
                        exchange: symbolItem.exchange,
                        minmov: 1,
                        pricescale: 100,
                        has_intraday: true,
                        has_no_volume: true,
                        has_empty_bars: false,
                        has_weekly_and_monthly: false,
                        supported_resolutions: configurationData.supported_resolutions,
                        volume_precision: 2,
                        data_status: 'streaming',
                    };
                    onSymbolResolvedCallback(symbolInfo);
                } catch (error) {
                    console.error("Error during resolveSymbol:", error);
                    onResolveErrorCallback('Error resolving symbol');
                }
            },
            getBars: async (symbolInfo, resolution, periodParams, onHistoryCallback, onErrorCallback) => {
                try {
                    const { from, to } = periodParams;
                    const bars = await getHistoricalData(handler, symbolInfo.ticker, from, to, resolution);
                    if (bars == null)
                        onHistoryCallback(new Array(), { noData: true });
                    else
                        onHistoryCallback(bars, { noData: false });
                } catch (error) {
                    console.error("Error during getBars:", error);
                }
            },
            subscribeBars: (symbolInfo, resolution, onRealtimeCallback, subscribeUID, onResetCacheNeededCallback) => {
                widgets[symbolInfo.ticker] = onRealtimeCallback;
                const symbols = getSymbols(symbolInfo.ticker);
                handler.invokeMethodAsync("Subscribe", symbols);
            },
            unsubscribeBars: (symbolInfo, subscriberUID) => {
                handler.invokeMethodAsync("Unsubscribe", symbolInfo.ticker);
                console.log('[unsubscribeBars]: Method call with subscriberUID:', subscriberUID);
            },
        }
    });
    charts[chartId] = chart;
    //if (state != null) {
    //    chart.onChartReady(() => {
    //        var config = JSON.parse(state);
    //        config.charts[0].panes[0].sources[0].state.symbol = symbol;

    //        chart.load(config);
    //    });
    //}
}

function getSymbols(ticker) {
    const operators = /[+\-*/]/g;
    return ticker.split(operators).map(x => x.trim()).filter(item => item !== '');
}

async function getHistoricalData(handler, ticker, from, to, resolution) {
    const symbols = getSymbols(ticker);
    const bars = await handler.invokeMethodAsync('GetData', symbols, from, to, resolution);

    if (bars == null || Object.keys(bars).length === 0)
        return null;

    if (Object.keys(bars).length === 1) return bars[ticker];

    const result = [];
    const expr = window.Parser.parse(ticker);
    for (let i = 0; i < bars[symbols[0]].length; i++) {
        const params = {
            time: 0, open: {}, close: {}, high: {}, low: {}
        };
        symbols.forEach(symbol => {
            const row = bars[symbol][i];
            params.time = row.time;
            params.close[symbol] = row.close;
            params.open[symbol] = row.open;
            params.high[symbol] = row.high;
            params.low[symbol] = row.low;
        });
        const buckets = {
            open: expr.evaluate(params.open),
            close: expr.evaluate(params.close),
            high: expr.evaluate(params.high),
            low: expr.evaluate(params.low),
            time: params.time
        }
        result.push(buckets);
    }
    return result;
}

function updateWidgetData(id, bars) {
    var callBack = widgets[id];
    if (callBack !== undefined) {
        if (bars.length == 1) {
            callBack(bars[0]);
        }
        else {
            var expr = Parser.parse(id);
            const params = {
                time: 0, open: {}, close: {}, high: {}, low: {}
            };
            bars.forEach((row) => {
                params.time = row.time;
                params.close[row.identifier] = row.close;
                params.open[row.identifier] = row.open;
                params.high[row.identifier] = row.high;
                params.low[row.identifier] = row.low;
            });
            const bucket = {
                open: expr.evaluate(params.open),
                close: expr.evaluate(params.close),
                high: expr.evaluate(params.high),
                low: expr.evaluate(params.low),
                time: params.time
            };
            callBack(bucket);
        }
    }
}

function getChartSettings(chartId) {
    var chart = charts[chartId];
    var all = {}
    if (chart !== undefined) {
        chart.save(function (data) {
            all = data;
        });
        return JSON.stringify(all);
    }
    return null;
}

const configurationData = {
    supported_resolutions: ['1', '5', '15', '30', '60', '240', 'D'],
    exchanges: [
        {
            value: 'Bitfinex',
            name: 'Bitfinex',
            desc: 'Bitfinex',
        },
    ]
};

// Default Multi Chart Implementation
async function createSeries(chartId, handler) {
    var chart = charts[chartId];
    if (chart === undefined)
        return;
    chart.precision = handler.precision;
    deleteSeries(chartId, handler.displayName);
    var minMove = 1 / (10 ** chart.precision);
    var lineOption =
    {
        //topColor: 'rgba(38, 198, 218, 0.10)',
        //bottomColor: 'rgba(38, 198, 218, 0.04)',
        color: handler.color,
        lineWidth: 1,
        crossHairMarkerVisible: false,
        priceLineVisible: false,
        priceFormat: {
            minMove: minMove,
            precision: chart.precision,
        },
    };
    if (handler.isLeftScale) {
        chart.applyOptions({
            leftPriceScale: {
                visible: true,
            },
        });
        lineOption.priceScaleId = 'left';
    }
    var areaSeries = chart.addSeries(LightweightCharts.LineSeries,lineOption);
    areaSeries.lineDatas = [];
    areaSeries.markerDatas = [];
    chart.series[handler.displayName] = areaSeries;
}

function delay(time) {
    return new Promise(resolve => setTimeout(resolve, time));
}

function setSeriesOption(chartId, seriesId, color) {
    var chart = charts[chartId];
    if (chart == undefined) return;
    var line = chart.series[seriesId];
    if (line == undefined) return;
    line.applyOptions({ 'color': color });
}


function deleteSeries(chartId, seriesId) {

    var chart = charts[chartId];
    if (chart == undefined) return;
    var line = chart.series[seriesId];
    if (line == undefined) return;

    chart.removeSeries(line);
    delete chart.series[seriesId];
}

function clearCharts() {
    for (var member in charts) {
        var chart = charts[member];
        chart.remove();
    }
    charts = {};
}

function historyData(chartId, symbol, data) {
    var chart = charts[chartId];
    if (chart == undefined) return;
    var sLine = chart.series[symbol];
    var oldDatas = sLine.lineDatas;
    if (sLine !== undefined && oldDatas !== undefined) {
        // Check for overlapping data points by some unique identifier (e.g., 'id') before concatenation

        var combinedData = data.concat(oldDatas);
        combinedData = filterData(combinedData);
        sLine.setData(combinedData);
        sLine.lineDatas = combinedData;
    }
}

function filterData(data) {
    var filterData = [...new Map(data.map(item => [item.id, item])).values()];
    //var filterData = data.filter(newPoint => !oldDatas.some(oldPoint => oldPoint.id === newPoint.id));
    filterData.sort((a, b) => a.id - b.id);
    return filterData;
}

function updateSeries(chartId, line, data) {
    var chart = charts[chartId];
    if (chart == undefined) return;

    var seriesLine = chart.series[line];
    if (seriesLine !== undefined && seriesLine.lineDatas !== undefined) {

        if (seriesLine.lineDatas.length > 0) {
            var lastId = seriesLine.lineDatas[seriesLine.lineDatas.length - 1].time;
            if (data.time <= lastId) {
                return; // Exit if data.id is not higher than the last ID
            }
        }
        seriesLine.lineDatas.push(data);
        seriesLine.update(data);
    }

}
function updateMarker(chartId, symbol, data) {
    var chart = charts[chartId];
    if (chart == undefined) return;
    var seriesLine = series[symbol];
    if (seriesLine !== undefined && seriesLine.markerDatas !== undefined) {
        seriesLine.markerDatas.push(data);
        LightweightCharts.createSeriesMarkers(seriesLine, seriesLine.markerDatas);
    }
}

function setData(id, symbol, data) {
    var chart = charts[id];
    if (chart == undefined) return;
    var seriesLine = chart.series[symbol];
    if (seriesLine == undefined) return;

    seriesLine.setData(data);
    seriesLine.lineDatas = data;
    chart.timeScale().scrollToPosition(10, false);
}
function setMarkers(id, symbol, data) {
    var chart = charts[id];
    if (chart == undefined) return;
    var seriesLine = chart.series[symbol];
    if (seriesLine == undefined) return;

    seriesLine.markerDatas = data;
    LightweightCharts.createSeriesMarkers(seriesLine, seriesLine.markerDatas);
}

function getRange(id) {
    var chart = charts[id];
    var timeScale = chart.timeScale();
    var logicalRange = timeScale.getVisibleLogicalRange();
    if (logicalRange != null)
        return logicalRange.from;
    else
        return 0;
}

function makeChart(id, dotnetChart) {
    var bdy = document.getElementById(id);

    var chart = LightweightCharts.createChart(bdy, {
        width: 700,
        height: 400,
        layout: {
            textColor: '#FFFFFF',
            background: { color: '#181A20' },
            fontfamily: 'Roboto',
            fontsize: '24'
        },
        rightPriceScale: {
            scaleMargins: {
                textColor: '#FFFFFF',
                top: 0.15,
                bottom: 0.05,
            },
        },
        crosshair: {
            vertLine: {
                visible: false,
                labelVisible: false,
            },
            horzLine: {
                visible: false,
                labelVisible: false,
            },
        },

        grid: {
            vertLines: {
                color: 'rgba(211, 211, 211, 0.05)', // light gray color with some transparency
                width: 0.5, // thin width
            },
            horzLines: {
                color: 'rgba(211, 211, 211, 0.05)', // light gray color with some transparency
                width: 0.5, // thin width
            },
        },
        timeScale: {
            timeVisible: true,
            secondsVisible: false,
        },
    });
    chart.series = {};
    charts[id] = chart;
    var timeScale = chart.timeScale();
    timeScale.subscribeVisibleLogicalRangeChange(() => {
        var logicalRange = timeScale.getVisibleLogicalRange();
        if (logicalRange != null)
            dotnetChart.invokeMethodAsync("LoadData", logicalRange.from, logicalRange.to);
    });

    const toolTipWidth = 180;
    const toolTipHeight = 80;
    const toolTipMargin = 15;

    // Create and style the tooltip html element
    var toolTip = document.getElementById(id + "_tooltip");
    var toolTipList = document.getElementById(id + "_container");
    if (toolTip !== undefined && toolTipList != undefined) {

        chart.subscribeCrosshairMove(param => {
            if (
                param.point === undefined || param.seriesData === undefined ||
                !param.time ||
                param.point.x < 0 ||
                param.point.x > bdy.clientWidth ||
                param.point.y < 0 ||
                param.point.y > bdy.clientHeight
            ) {
                toolTip.style.display = 'none';
            } else {
                var d = new Date(param.time * 1000);
                toolTip.children[0].innerHTML = d.toUTCString();

                var precision = 2;
                var valueElements = [];
                if (chart.precision != undefined) {
                    precision = chart.precision;
                }
                for (const sLine in chart.series) {
                    var lineElem = document.getElementById(id + '_' + sLine);
                    if (lineElem == undefined) continue;

                    var data = param.seriesData.get(chart.series[sLine]);
                    if (data != undefined) {
                        lineElem.children[0].innerHTML = data.value.toFixed(precision);
                    }
                    valueElements.push({
                        value: lineElem.children[0].innerHTML,
                        element: lineElem
                    });
                }
                if (valueElements.length > 1) {
                    valueElements.sort((a, b) => b.value - a.value); // Ascending order
                    // Remove and re-append the elements in the new order
                    valueElements.forEach(item => {
                        toolTipList.append(item.element);
                    });
                }

                const y = param.point.y;
                let left = param.point.x + toolTipMargin;
                if (left > bdy.clientWidth - toolTipWidth) {
                    left = param.point.x - toolTipMargin - toolTipWidth;
                }

                let top = y + toolTipMargin;
                if (top > bdy.clientHeight - toolTipHeight) {
                    top = y - toolTipHeight - toolTipMargin;
                }

                toolTip.style.display = 'block';
                toolTip.style.left = left + 'px';
                toolTip.style.top = top + 'px';
            }
        });
    }
    new ResizeObserver(function () {
        if (chart !== null)
            chart.applyOptions({ width: bdy.offsetWidth, height: bdy.offsetHeight });
    }).observe(bdy)
}

function scale(number, inMin, inMax, outMin, outMax) {
    return (number - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

//Plotly treemap
function plotTreemap(id, labels, parents, values, absValues) {
    var cmin = Math.min(...values);  // Set the minimum of the color scale
    var cmax = Math.max(...values);  // Set the maximum of the color scale

    

    //(x-min(x))/(max(y)- min(y))
    // Normalize the values to a 0-100 scale
    const normalizedValues = values.map(value => {
        if (value < 0) {
            // For negative values, map them from -1 to 0
            return scale(value, cmin, 0, 0, 1);
        } else {
            // For positive values, map them from 0 to 1
            return scale(value, 0, cmax, 1, 2);
        }
    });
    // Create the treemap
    const trace = [{
        type: "treemap",
        labels: labels,
        parents: parents,
        values: absValues,
        customdata: values,
        pathbar: { "visible": false },
        hovertemplate: "%{label}<br> Total %{customdata} <extra></extra>",
        //texttemplate: "%{label} %{customdata}<br> (%{percentTotal})",
        branchvalues: "total",
        textinfo: "label+customdata+percent entry",
        marker: {
            "line": { "color": "black", "width": 0.5 },
            //colors: ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd'],
            colors: normalizedValues,  // Apply colors based on the actual data values
            autocolorscale: false,      // Disable automatic color scaling
            reversescale: false,         // Keep green for positive, red for negative
            colorscale: [
                [0, '#D32F2F'],  // Red (for negative values),
                [0.48, '#EF5350'],
                [0.5, 'white'],
                [0.52, '#66BB6A'],
                [1, '#43A047']  // Green (for positive values)
            ],
        },

    }];


    const child = document.getElementById(id);
    const parent = child.parentElement; // Get the parent of the child

    child.style.height = (parent.offsetHeight - 45) + "px";
    const layout = {
        paper_bgcolor: "black",
        width: parent.offsetWidth,
        height: parent.offsetHeight - 45,
        margin: { l: 0, r: 0, t: 0, b: 0 }
    };
    var div = document.getElementById(id);
    if (div.innerHTML == '') {
        Plotly.newPlot(id, trace, layout);
        new ResizeObserver(function () {
            Plotly.relayout(id, {
                width: parent.offsetWidth,
                height: parent.offsetHeight - 45
            });
        }).observe(div.parentElement);
    }
    else {
        Plotly.react(id, trace, layout);
    }
   
    //Plotly.relayout(id, {
    //    width: parent.offsetWidth,
    //    height: parent.offsetHeight - 45
    //});
}
