﻿const goldenInstances = {};
const goldenConfig = {};
const mouse_point = {};
const linkedPanels = [
    "BadgerWeb.Pages.DockApp.Instruments.InstrumentPanel",
    "BadgerWeb.Pages.DockApp.Charts.MultiChartPanel",
    "BadgerWeb.Pages.DockApp.Charts.IndicatorChartPanel",
    "BadgerWeb.Pages.DockApp.Orders.OrderBookPanel",
    "BadgerWeb.Pages.DockApp.Strategies.StrategiesPanel",
    "BadgerWeb.Pages.DockApp.Strategies.PortfolioPanel"
];

document.addEventListener("mousemove", function (event) {
    mouse_point.x = event.clientX;
    mouse_point.y = event.clientY;
});

function createGoldenLayout(id, config, panelRef) {
   
    var mConfig = {
        settings: {
            hasHeaders: true,
        },
        dimensions: {
            borderWidth: 5,
        },
        content: [{
            type: 'row',
            content: [{
                type: 'component',
                componentName: 'idComponent',
                componentState: { id: 'Temp' }
            }]
        }]
    };
    if (config != null) {
        mConfig = JSON.parse(config);
    }
    mConfig.settings.showMaximiseIcon = false;
    mConfig.settings.showPopoutIcon = false;
    mConfig.settings.selectionEnabled = true;
    mConfig.dimensions.borderWidth = 8;
    mConfig.dimensions.headerHeight = 28;
    var layoutElement = document.getElementById(id);
    var myLayout = new GoldenLayout(mConfig, layoutElement);
    myLayout.registerComponent("idComponent", function (container, componentState) {
        var myElement = document.getElementById(componentState.id);
        if (myElement !== null) {
            container.setTitle(componentState.title);
            container.getElement().html(myElement);
            
        }
    });
    myLayout.on("itemDestroyed", function (data) {
        if (data.config.id != null) {
            panelRef.invokeMethodAsync("Destroy", data.config.id);
        }
    });
    myLayout.on('stackCreated', function (stack) {

        //HTML for the colorDropdown is stored in a template tag
        var colorDropdown = $($('template').html());
        var colorDropdownBtn = colorDropdown.find('.selectedColor');


        var setColor = function (color) {
            var contentItem = stack.getActiveContentItem();
            var container = contentItem.container;

            if (linkedPanels.includes(contentItem.config.componentState.panelType)) {
                colorDropdown.css('display', 'block')
            }else {
                colorDropdown.css('display', 'none')
            }
            var hexedColor = rgbToHex(color);
            if (hexedColor == '#808080' || hexedColor == 'grey')
                color = 'inherit';
            // Update the state
            colorDropdownBtn.css('background-color', color);
            contentItem.config.componentState.state = hexedColor;
            container.extendState({ color: color });
        };

        // Add the colorDropdown to the header
        stack.header.controlsContainer.prepend(colorDropdown);

        // Update the color initially and whenever the tab changes
        stack.on('activeContentItemChanged', function (contentItem) {
            if (contentItem.config.componentState.state != undefined) {
                setColor(contentItem.config.componentState.state);
            }
        });

        // Update the color when the user selects a different color
        // from the dropdown
        colorDropdown.find('li').click(function () {
            var color = $(this).css('background-color');
            var contentItem = stack.getActiveContentItem();
            
            panelRef.invokeMethodAsync("OnStateUpdated", contentItem.config.id, rgbToHex(color));
            setColor(color);
        });
    });
    myLayout.init();
    goldenInstances[id] = myLayout;
    window.addEventListener('resize', () => myLayout.updateSize());
    //window.addEventListener('resize', () => myLayout.updateSize(window.innerWidth - 5, window.innerHeight - 70));
}

function rgbToHex(rgb) {
    if (rgb.startsWith('rgb') == false) return rgb;
    var a = rgb.split("(")[1].split(")")[0];
    a = a.split(",");
    var b = a.map(function (x) {
        x = parseInt(x).toString(16);
        return (x.length == 1) ? "0" + x : x;
    });
    return "#" + b.join("");
}

function updateName(id, containerId, title) {
    var myLayout = goldenInstances[id];
    if (myLayout !== undefined) {
        var component = myLayout.root.contentItems[0].getItemsById(containerId);
        if (component[0] !== undefined) {
            component[0].setTitle(title);
        }
    }
}

function destroy(id) {
    if (goldenInstances[id] !== undefined) {
        goldenInstances[id].destroy();
        goldenInstances[id] = null;
    }
}

function addLayout(id, config) {
    if (goldenInstances[id] === undefined) {
        return;
    }
    var newItemConfig = JSON.parse(config);
    var mLayout = goldenInstances[id];
    if (mLayout.selectedItem != null)
        mLayout.selectedItem.addChild(newItemConfig);
    else
        mLayout.root.contentItems[0].addChild(newItemConfig);
}

function saveLayout(id) {
    if (goldenInstances[id] === undefined) {
        return false;
    }
    else {
        var state = JSON.stringify(goldenInstances[id].toConfig());
        goldenConfig[id] = state;
        return true;
    }
}

function getConfig(id, start) {
    if (goldenConfig[id] === undefined) {
        return null;
    }
    var configData = goldenConfig[id];
    if (configData.length <= start) {
        delete goldenConfig[id];
        return null;
    }
        
    var maxByte = 20 * 1024;
    var end = start + maxByte;
    if (configData.length < end)
        return configData.substring(start);
    else {
        var data = configData.substring(start, end);
        return data;
    }
}

function removeElement(elementId) {
    var element = document.getElementById(elementId);
    if (element) {
        element.remove();
    }
}
function dragElement(elementId) {
    var pos1 = 0, pos2 = 0, pos3 = 0, pos4 = 0;
    var elmnt = document.getElementById(elementId);
    
    if (elmnt) {
        var height = elmnt.parentNode.parentElement.clientHeight;
        var width = elmnt.parentNode.parentElement.clientWidth;
        var top = Math.min(Math.max(mouse_point.y, 55), height - elmnt.clientHeight);
        var left = Math.min(Math.max(mouse_point.x, 10), width - elmnt.clientWidth);
        elmnt.style.top = top + "px";
        elmnt.style.left = left + "px";
        if (document.getElementById(elmnt.id + "header")) {
            /* if present, the header is where you move the DIV from:*/
            document.getElementById(elmnt.id + "header").onmousedown = dragMouseDown;
        } else {
            /* otherwise, move the DIV from anywhere inside the DIV:*/
            elmnt.onmousedown = dragMouseDown;
        }
    }
    function dragMouseDown(e) {
        e.preventDefault();
        // get the mouse cursor position at startup:
        pos3 = e.clientX;
        pos4 = e.clientY;
        document.onmouseup = closeDragElement;
        // call a function whenever the cursor moves:
        document.onmousemove = elementDrag;
    }

    function elementDrag(e) {
        e.preventDefault();
        // calculate the new cursor position:
        pos1 = pos3 - e.clientX;
        pos2 = pos4 - e.clientY;
        pos3 = e.clientX;
        pos4 = e.clientY;
        // set the element's new position:
        var top = Math.min(Math.max(elmnt.offsetTop - pos2, 55), height - 40);
        var left = Math.min(Math.max(elmnt.offsetLeft - pos1, 10), width - elmnt.clientWidth)
        elmnt.style.top = top + "px";
        elmnt.style.left = left + "px";
    }

    function closeDragElement() {
        /* stop moving when mouse button is released:*/
        document.onmouseup = null;
        document.onmousemove = null;
    }
}