"use strict";
var lockedGrid;
var lockedRecord;
async function downloadFileFromStream(fileName, contentStreamReference) {
	const arrayBuffer = await contentStreamReference.arrayBuffer();
	const blob = new Blob([arrayBuffer]);

	const url = URL.createObjectURL(blob);

	const anchorElement = document.createElement('a');
	anchorElement.download = fileName ?? '';
	anchorElement.href = url;
	anchorElement.click();
	anchorElement.remove();

	URL.revokeObjectURL(url);
}

function triggerFileDownload(url) {
	const anchorElement = document.createElement('a');
	anchorElement.href = url;
	anchorElement.click();
	anchorElement.remove();
}
w2utils.formatters['progress_circle'] = function (val) {
	var color = "mud-primary-text";
	if (val < 0) {
		val = Math.abs(val);
		color = "mud-error-text";
	}
	var out = Math.trunc(val);
	var subData =
		'<div class= "d-flex mx-auto justify-content-center align-items-center">' +
		`<div style="height:24px;width:24px;opacity:0.5" class="mud-progress-circular mud-progress-static position-absolute mud-tertiary-text" ` +
		`role="progressbar" aria-valuenow="0.2">` +
		'<svg class="mud-progress-circular-svg" viewBox="22 22 44 44">' +
		'<circle class="mud-progress-circular-circle mud-progress-static" ' +
		'cx="44" cy="44" r="20" fill="none" stroke-width="5" ' +
		`style="stroke-dasharray: 126; stroke-dashoffset: 0;"></circle></svg></div >` +
		`<div style="height:24px;width:24px" class="mud-progress-circular mud-progress-static position-absolute ${color}" ` +
		`role="progressbar" aria-valuenow="0.2">` +
		'<svg class="mud-progress-circular-svg" viewBox="22 22 44 44">' +
		'<circle class="mud-progress-circular-circle mud-progress-static" ' +
		'cx="44" cy="44" r="20" fill="none" stroke-width="5" ' +
		`style="stroke-dasharray: 126; stroke-dashoffset: ${126 - val};"></circle></svg></div>`;
	if (val < 100) {
		subData = subData + `<span class="${color} fs-8 font-weight-bold">${out}</span>`;
	}
	subData = subData + "</div >";
	return subData;
}

w2utils.formatters['progress_function'] = function (val, params, record) {
	var perc = val * 100 / 4;
	var nPerc = Math.abs(Math.min(perc, 0));
	var pPerc = Math.max(perc, 0);
	return `<div title="Score: ${val}" class="d-flex flex-row"><div class="progress reverse_progress w-50 z_score_progress_red_background">` +
		'<div class="progress-bar z_score_progress_red" role="progressbar" aria-valuenow="20" aria-valuemin="0" aria-valuemax="4" ' +
		' style="width:' + nPerc + '%">' +
		'</div></div><div class="progress z_score_progress_green_background w-50" style="width:50%">' +
		'<div class="progress-bar z_score_progress_green" role="progressbar" aria-valuenow="20" aria-valuemin="0" aria-valuemax="4" ' +
		'style="width:' + pPerc + '%">' +
		'</div></div></div>';
}

w2utils.formatters['cross_order_function'] = function (val, params, record) {
	return `<div class=circular_state style=\"${record.CrossOrderState}\"></div>`;
}

w2utils.formatters['dynamic_float'] = function (val, params, record) {
	if (record.PriceTickLength != undefined && record.PriceTickLength > 0)
		return val.toFixed(record.PriceTickLength);
	else
		return val;
}

w2utils.formatters['basis_avatar'] = function (val, params, record) {
	return '<span class="d-flex flex-row">' +
		`<img style="margin:0 3px" width="15" height="15" src=${record.icon} />` +
		`${record.name}` +
		`</span>`;
}
w2utils.formatters['strategy_icon_buy'] = function (val, params, record, data) {
	if (val == true) {
		return `<span title="${record.BuyState}" data-action="BuyLeg" style="font-size:18px" class="material-icons">circle</span>`;
	}
	else {
		return `<span title="${record.BuyState}" data-action="BuyLeg" style="font-size:18px" class="material-icons md-18">blur_circular</span>`;
	}
}
w2utils.formatters['strategy_icon_sell'] = function (val, params, record, data) {
	if (val == true) {
		return `<span title="${record.SellState}" data-action="SellLeg" style="font-size:18px" class="material-icons md-18">circle</span>`;
	}
	else {
		return `<span title="${record.SellState}" data-action="SellLeg" style="font-size:18px" class="material-icons md-18">blur_circular</span>`;
	}
}

function handleClick(recordId, gridId, action) {
	var grid = w2ui[gridId];
	var handler = w2ui[gridId + '_handler'];
	if (grid !== undefined && handler != null) {
		handler.invokeMethodAsync('HandleButtonActionClick', action, recordId);
    }
	
}

window.BlazorUIGrid = {
	createGrid: function (id, dotnetRef, options, headerMenus) {
		options.columns.forEach(x => {
			if (x.renderFunction) {
				x.render = new Function('record','extra', x.render);
			}
		});
		

		
		w2ui[id + '_handler'] = dotnetRef;
		$('#' + id).w2grid(options);
		var grid = w2ui[id];
		grid.on('contextMenu', function (event) {
			if (event.detail.recid == null || event.detail.recid == '-none-') {
				event.preventDefault();
				var div = document.getElementById('grid_' + id + '_columns');
				if (div != null && div.oncontextmenu == null) {
					div.oncontextmenu = function contextData(event) {
						$('body').w2menu({
							items: headerMenus,
							onSelect(event) {
								grid.selectNone();
								dotnetRef.invokeMethodAsync('HandleHeaderActionClick', event.detail.item.id);
							},
							originalEvent: event
						});
						event.preventDefault();
					};
				}
			}
		});
		grid.on('menuClick', function (event) {
			dotnetRef.invokeMethodAsync('HandleMenuActionClick', event.detail.menuItem.id, event.detail.recid.toString());
		});
		grid.on('editField', function (event) {
			lockedGrid = id;
			lockedRecord = event.detail.recid;
		});
		grid.on('change', function (event) {
			event.onComplete = async function () {

				await new Promise(resolve => setTimeout(resolve, 20));
				grid.save();
				var data = grid.get(event.detail.recid);
				var changedField = {};
				options.columns.forEach(x => {
					if (x.editable.type !== undefined) {
						changedField[x.field] = data[x.field];
					}
				});
				var request = {
					RecId: event.detail.recid,
					FieldDatas: changedField
				};
				await dotnetRef.invokeMethodAsync('OnEditRequested', request);
				lockedGrid = lockedRecord = undefined;
			}
		});

		grid.on('dblClick', function (event) {
			event.detail.originalEvent.data = 'Handled';
			lockedGrid = lockedRecord = undefined;
			var dt = grid.columns[event.detail.column];
			if (dt.editable.type == undefined) {
				
				var clientX = event.detail.originalEvent.originalEvent.clientX;
				var clientY = event.detail.originalEvent.originalEvent.clientY;
				dotnetRef.invokeMethodAsync('HandleDblActionClick', event.detail.recid, clientX, clientY);
			}
		});
		var div = document.getElementById(id);
		if (div != undefined) {
			new ResizeObserver(function () {
				grid.refresh();
			}).observe(div)
		}
		

		grid.refresh();
	},


	callApi: function (id, functionName, data) {
		if (data == null) {
			w2ui[id][functionName]();
		}
		else {
			w2ui[id][functionName](data);
		}
	},
	getData: function (id, recId) {
		var grid = w2ui[id];
		if (grid === undefined) {
			return null;
		}
		var oldData = grid.get(recId);
		return oldData;
	},
	getRecordIds: function (id) {
		var grid = w2ui[id];
		if (grid === undefined) {
			return [];
		}
		var ids = grid.records.map(a => a[grid.recid]);
		return ids;
	},
	addData: function (id, jsonData) {
		var data = JSON.parse(jsonData);
		w2ui[id].add(data);
	},
	insertData: function (id, jsonData) {
		var datas = JSON.parse(jsonData);
		w2ui[id].add(datas, true);
	},
    addUpdateData: function (id, jsonData) {
        var grid = w2ui[id];
        if (grid === undefined) {
            return;
        }
		var data = JSON.parse(jsonData);
		var oldData = grid.get(data[grid.recid]);
		if (oldData == null)
			grid.add(data, true);
		else
			grid.set(data[grid.recid], data);

	},
    addUpdateDatas: function (id, jsonData) {
        var grid = w2ui[id];
        if (grid === undefined) {
            return;
        }
        var datas = JSON.parse(jsonData);
        datas.forEach(data => {
            var oldData = grid.get(data[grid.recid]);
            if (oldData == null)
                grid.add(data, true);
            else
                grid.set(data[grid.recid], data);
        });
    },
	updateData: function (id, jsonData, refreshFull) {
		var grid = w2ui[id];
		if (grid === undefined) {
			return;
		}
		var data = JSON.parse(jsonData);
		if (id == lockedGrid) {
			data.forEach(x => {
				if (x[grid.recid] != lockedRecord)
					grid.set(x[grid.recid], x);
			});
		}
		else {
			data.forEach(x => {
				grid.set(x[grid.recid], x, true);
			});
			if (refreshFull) {
				grid.update({ fullCellRefresh: true });
			}
			else {
				grid.update();
			}
		}
	},
	deleteDatas: function (id, jsonData) {
		var grid = w2ui[id];
		if (grid !== undefined) {
			var datas = JSON.parse(jsonData);
			datas.forEach(data => grid.remove(data));
		}
	},
	setData: function (id, jsonData) {
		var grid = w2ui[id]; 
		if (grid !== undefined) {
			var data = JSON.parse(jsonData);
			grid.clear();
			grid.records = data;
			grid.refresh();
			//grid.render();
		}
	},
	setSummary: function (id, jsonData) {
		var grid = w2ui[id];
		if (grid !== undefined) {
			var data = JSON.parse(jsonData);
			data.forEach(x => {
				grid.set(x[grid.recid], x);
			});
		}
	},
	getState: function (id) {
		var jsonState = w2ui[id].stateSave(true);
		return JSON.stringify(jsonState);
	},

	setState: function (id, state) {
		var jsonState = JSON.parse(state);
		jsonState.show = null;
		w2ui[id].stateRestore(jsonState);
	},

	
}

const connections = {};
const gridIds = {};
function subscribeGrid(name, id) {
	if (gridIds[name] == undefined) {
		gridIds[name] = [];
	}
	if (gridIds[name].includes(id) == false) {
		gridIds[name].push(id);
	}
}

function unSubscribeGrid(id) {
    for (const name in gridIds) {
        const index = gridIds[name].indexOf(id);
        if (index > -1) {
            gridIds[name].splice(index, 1);
            // If the array is empty, you may want to delete the key
            if (gridIds[name].length === 0) {
                delete gridIds[name];
            }
        }
    }
}


function reconnect() {
	Object.entries(myObject).forEach(([key, connection]) => {
		if (connection != undefined && connection.state() != 'Connected') {
			connection.start();
		}
	})
	
}

async function openConnection(name, url, token) {
	var connection = connections[name];
    if (connection == undefined) {
        connection = new signalR.HubConnectionBuilder()
            .withAutomaticReconnect()
            .withUrl(url, { accessTokenFactory: () => token })
			.build();

		await connection.start();
		connections[name] = connection;

		connection.onreconnecting((error) => {
			console.warn("Reconnecting...", error);
		});

		connection.onclose((error) => {
			console.error("Connection closed.", error);
		});

		connection.onreconnected((connectionId) => {
			console.log("Reconnected");
			// Reload the page when reconnected
			window.location.reload();
		});
        connection.on("OnInstrumentUpdate", function (data) {
            var insIds = gridIds["Instrument"];
            updateBatchData(insIds, data, false);
        });
        connection.on("OnStrategyUpdate", function (data) {
            var strategyIds = gridIds["Strategy"];
            updateBatchData(strategyIds, data, false);
        });
        connection.on("OnPortfolioUpdate", function (data) {
            var portfolioIds = gridIds["Portfolio"];
            updateBatchData(portfolioIds, data, true);
        });

        connection.on("OnTickerUpdate", function (data) {
            var id = data[0].Identifier;
            if (id != undefined) {
                id = "Ticker_" + id;
                var grids = gridIds[id];
                addBatchData(grids, data, true);
            }
		});
		connection.invoke("OpenLive");
		
	}
	return connection.connectionId;
}

function liveConnectionCommand(name, method, data) {
	var connection = connections[name];
	if (connection == undefined) {
		return null;
	}
	return connection.invoke(method, data);
}

function addBatchData(ids, data, addTop) {
	if (ids == undefined) return;
    ids.forEach(id => {
        var grid = w2ui[id];
        if (grid != undefined) {
            grid.add(data, addTop);
        }
    });
}

function focusElement(id) {
	var elem = document.getElementById(id);
	if (elem == undefined) return;
	elem.focus();
}

function updateBatchData(ids, datas, refreshFull) {
	if (ids == undefined) return;
	//var datas = JSON.parse(jsonData);
	ids.forEach(id => {
		var grid = w2ui[id];
		if (grid != undefined) {
			if (id == lockedGrid) {
				datas.forEach(x => {
					if (x[grid.recid] != lockedRecord)
						grid.set(x[grid.recid], x);
				});
			} else {
				datas.forEach(x => {
					grid.set(x[grid.recid], x, true);
				});
				if (refreshFull)
					grid.update({ fullCellRefresh: true });
				else
					grid.update();
			}
		}
	});
}

function makeOrderbookTable(tableId, dotnetTable) {
	var table = document.getElementById(tableId);
	table.addEventListener('wheel', function (ev) {
		dotnetTable.invokeMethodAsync("OnScroll", ev.deltaY);
	});
	new ResizeObserver(function () {
		dotnetTable.invokeMethodAsync("OnSize", table.clientWidth, table.clientHeight);
	}).observe(table);
}

function scrollToElement(containerId, posPerc) {
	const container = document.getElementById(containerId);
	if (container && container.children.length > 0) {
		const table = container.children[0];
		const topPos = table.offsetTop - (container.clientHeight / 2) + (table.clientHeight * posPerc);
		//const topPos = element.offsetTop - (container.clientHeight / 2) + (element.clientHeight / 2);
		container.scrollTop = topPos;
	}
}

function googleRecaptcha(dotNetObject, selector, sitekeyValue) {
	return grecaptcha.render(selector, {
		'sitekey': sitekeyValue,
		'callback': (response) => { dotNetObject.invokeMethodAsync('CallbackOnSuccess', response); },
		'expired-callback': () => { dotNetObject.invokeMethodAsync('CallbackOnExpired', response); }
	});
};

function getResponse(response) {
	return grecaptcha.getResponse(response);
} 